/** AxisLabels.java.

	Purpose:
		
	Description:
		
	History:
		17:52:02 PM Jan 13, 2014, Created by 

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */
package org.zkoss.chart;

import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * The axis labels show the number or category for each tick.
 * 
 * <p> All the options in this class support {@link DynamicalAttribute}.
 * @author jumperchen
 * @author RaymondChao
 */
public class AxisLabels extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		align,
		autoRotation,
		autoRotationLimit,
		distance,
		enabled,
		format,
		formatter,
		maxStaggerLines, //deprecated
		overflow, //deprecated
		padding,
		rotation,
		staggerLines,
		step,
		style,
		useHTML,
		x,
		y,
		zIndex
	}

	/**
	 * Returns what part of the string the given position is anchored to. Can be
	 * one of <code>left</code>, <code>center</code> or <code>right</code>.
	 * <p> Defaults: <code>center</code>.
	 */
	public String getAlign() {
		return getAttr(Attrs.align, "center").asString();
	}

	/**
	 * Sets what part of the string the given position is anchored to. Can be
	 * one of <code>left</code>, <code>center</code> or <code>right</code>.
	 * <p> Defaults: <code>center</code>.
	 */
	public void setAlign(String align) {
		if (!("center".equals(align) || "left".equals(align) || "right".equals(align)))
			throw new IllegalArgumentException("Unsupported align [" + align + "]");
		setAttr(Attrs.align, align, "center");
	}

	/**
	 * For horizontal axes, the allowed degrees of label rotation to prevent
	 * overlapping labels. If there is enough space, labels are not rotated. As
	 * the chart gets narrower, it will start rotating the labels -45 degrees,
	 * then remove every second label and try again with rotations 0 and -45
	 * etc.
	 * <p>
	 * Defaults to [-45].
	 * 
	 * @since 2.1.0
	 */
	public List<Number> getAutoRotation() {
		return Generics.cast(getAttr(Attrs.autoRotation, Arrays.asList(new int[]{-45})).asValue());
	}
	/**
	 * Set the list of rotation degrees for horizontal axis labels. This list of
	 * degrees define all the possible rotation angles.
	 * @param autoRotation
	 * @see #getAutoRotation()
	 * @since 2.1.0
	 */
	public void setAutoRotation(List<Number> autoRotation) {
		setAttr(Attrs.autoRotation, autoRotation);
	}
	/**
	 * Set one or many rotation degrees for horizontal axis labels.
	 * @param autoRotation
	 * @see #setAutoRotation(List)
	 * @since 2.1.0
	 */
	public void setAutoRotation(Number... autoRotation) {
		setAttr(Attrs.autoRotation, Arrays.asList(autoRotation));
	}
	
	/**
	 * When each category width is more than this many pixels, we don't apply
	 * auto rotation. Instead, we lay out the axis label with word wrap. A lower
	 * limit makes sense when the label contains multiple short words that don't
	 * extend the available horizontal space for each label.
	 * <p>
	 * Defaults to 80.
	 * @return threshold where auto rotation starts to apply
	 * @since 2.1.0
	 */
	public Number getAutoRotationLimit() {
		return getAttr(Attrs.autoRotationLimit, 80).asNumber();
	}
	
	/**
	 * Set the width limit (in pixels) where auto rotation should stop being applied.
	 * @param autoRotationLimit
	 * @see #getAutoRotationLimit()
	 * @since 2.1.0
	 */
	public void setAutoRotationLimit(Number autoRotationLimit) {
		setAttr(Attrs.autoRotationLimit, autoRotationLimit);
	}
	
	/**
	 * Returns the distance of axis labels for Gauge chart type only.
	 * <p> Default: 15
	 */
	public Number getDistance() {
		return getAttr(Attrs.distance, 15).asNumber();
	}

	/**
	 * Sets the distance of axis labels for Gauge chart type only.
	 */
	public void setDistance(Number distance) {
		setAttr(Attrs.distance, distance, 15);
	}

	/**
	 * Returns whether to enable or disable the axis labels.
	 * <p>
	 * Default: true.
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, true).asBoolean();
	}

	/**
	 * Sets whether to enable or disable the axis labels.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Returns a <a href="http://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting">format
	 * string</a> for the axis label.
	 * <p>
	 * Default: <code>{value}</code>.
	 */
	public String getFormat() {
		return getAttr(Attrs.format, "{value}").asString();
	}

	/**
	 * Sets a <a href="http://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting">format string</a>
	 * for the axis label.
	 */
	public void setFormat(String format) {
		setAttr(Attrs.format, format, "{value}");
	}

	/**
	 * Callback JavaScript function to format the label. The value is given by
	 * "this.value". Additional properties for this are "axis", "chart", "isFirst" and
	 * "isLast".
	 * <p>
	 * Default: <pre>function() { return this.value; }</pre>
	 * @since 2.1.0
	 */
	public JavaScriptValue getFormatter() {
		return (JavaScriptValue) getAttr(Attrs.formatter, new JavaScriptValue("function() { return this.value; }")).asValue();
	}
	/**
	 * @param formatter
	 * @see #getFormatter()
	 * @since 2.1.0
	 */
	public void setFormatter(JavaScriptValue formatter) {
		setAttr(Attrs.formatter, formatter);
	}
	
	/**
	 * Returns the maximum lines of the to add to automatically avoid
	 * overlapping X labels. When <code>staggerLines</code> is not set,
	 * <code>maxStaggerLines</code> defines how many lines the axis is allowed
	 * to add to automatically avoid overlapping X labels. Set to <code>1</code>
	 * to disable overlap detection.
	 * <p>
	 * Default: 5.
	 */
	@Deprecated
	public Number getMaxStaggerLines() {
		return getAttr(Attrs.maxStaggerLines, 5).asNumber();
	}

	/**
	 * Sets the maximum lines of the to add to automatically avoid overlapping X
	 * labels. When <code>staggerLines</code> is not set,
	 * <code>maxStaggerLines</code> defines how many lines the axis is allowed
	 * to add to automatically avoid overlapping X labels. Set to <code>1</code>
	 * to disable overlap detection.
	 */
	@Deprecated
	public void setMaxStaggerLines(Number maxStaggerLines) {
		setAttr(Attrs.maxStaggerLines, maxStaggerLines, 5);
	}

	/**
	 * Returns how to handle overflowing labels on horizontal axis. Can be
	 * undefined or "justify". If "justify", labels will not render outside the
	 * plot area. If there is room to move it, it will be aligned to the edge,
	 * else it will be removed.
	 * <p>
	 * Default: null.
	 */
	@Deprecated
	public String getOverflow() {
		return getAttr(Attrs.overflow, null).asString();
	}

	/**
	 * Sets how to handle overflowing labels on horizontal axis. Can be
	 * undefined or "justify". If "justify", labels will not render outside the
	 * plot area. If there is room to move it, it will be aligned to the edge,
	 * else it will be removed.
	 * 
	 * @param overflow
	 *            how to handle overflowing labels on horizontal axis
	 */
	@Deprecated
	public void setOverflow(String overflow) {
		setAttr(Attrs.overflow, overflow);
	}

	/**
	 * The pixel padding for axis labels, to ensure white space between them.
	 * <p>
	 * Defaults to 5.
	 * @return the padding size in pixels
	 * @since 2.1.0
	 */
	public Number getPadding() {
		return getAttr(Attrs.padding, 45).asNumber();
	}
	/**
	 * Set the pixel padding for axis labels
	 * @param padding
	 * @see #getPadding()
	 * @since 2.1.0
	 */
	public void setPadding(Number padding) {
		setAttr(Attrs.padding, padding);
	}
	
	/**
	 * Returns rotation of the labels in degrees.
	 * <p>
	 * Default: null.
	 */
	public Object getRotation() {
		return getAttr(Attrs.rotation, null).asValue();
	}

	/**
	 * Sets rotation of the labels in degrees.
	 * 
	 * @param rotation
	 *            rotation of the labels in degrees
	 */
	public void setRotation(Number rotation) {
		setAttr(Attrs.rotation, rotation);
	}

	/**
	 * Sets rotation of the labels in degrees or <code>auto</code>.
	 * 
	 * @param rotation
	 *            rotation of the labels in degrees
	 */
	public void setRotation(String rotation) {
		setAttr(Attrs.rotation, rotation);
	}

	/**
	 * Returns the number of lines to spread the labels over to make room or
	 * tighter labels.
	 * <p>
	 * Default: 0
	 * 
	 * @return staggerLines the number of lines to spread the labels
	 */
	public Number getStaggerLines() {
		return getAttr(Attrs.staggerLines, 0).asNumber();
	}

	/**
	 * Sets the number of lines to spread the labels over to make room or
	 * tighter labels.
	 */
	public void setStaggerLines(Number staggerLines) {
		setAttr(Attrs.staggerLines, staggerLines, 0);
	}

	/**
	 * Returns to show only every <em>n</em>'th label on the axis, set the step to
	 * <em>n</em>. Setting the step to 2 shows every other label.
	 * </p>
	 * <p>
	 * By default, on a vertical axis, the step is calculated automatically to
	 * avoid overlap. To prevent this, set it to 1. This usually only happens on
	 * a category axis, and is often a sign that you have chosen the wrong axis
	 * type. Read more at <a
	 * href="http://www.highcharts.com/docs/chart-concepts/axes">Axis docs</a>
	 * => What axis should I use?
	 * </p>
	 * <p>
	 * Default: null.
	 */
	public Number getStep() {
		return getAttr(Attrs.step, null).asNumber();
	}

	/**
	 * Sets to show only every <em>n</em>'th label on the axis, set the step to
	 * <em>n</em>. Setting the step to 2 shows every other label.
	 * </p>
	 * <p>
	 * By default, on a vertical axis, the step is calculated automatically to
	 * avoid overlap. To prevent this, set it to 1. This usually only happens on
	 * a category axis, and is often a sign that you have chosen the wrong axis
	 * type. Read more at <a
	 * href="http://www.highcharts.com/docs/chart-concepts/axes">Axis docs</a>
	 * => What axis should I use?
	 * </p>
	 */
	public void setStep(Number step) {
		setAttr(Attrs.step, step);
	}

	/**
	 * Returns CSS styles for the label. When labels are rotated they are
	 * rendered using vector graphic techniques and not all styles are
	 * applicable.
	 * <p>
	 * Default:
	 * <pre>
	 * color: #6D869F; fontWeight: bold;
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("color: #6D869F; fontWeight: bold;");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Sets CSS styles for the label. When labels are rotated they are rendered
	 * using vector graphic techniques and not all styles are applicable.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the label. When labels are rotated they are rendered
	 * using vector graphic techniques and not all styles are applicable.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style, NOT_NULL_VALUE);
	}

	/**
	 * Returns whether to <a
	 * href="http://docs.highcharts.com/#formatting$html">use HTML</a> to render
	 * the labels.
	 * <p>
	 * Default: false.
	 */
	public boolean isUseHTML() {
		return getAttr(Attrs.useHTML, false).asBoolean();
	}

	/**
	 * Sets whether to <a href="http://docs.highcharts.com/#formatting$html">use
	 * HTML</a> to render the labels.
	 */
	public void setUseHTML(boolean useHTML) {
		setAttr(Attrs.useHTML, useHTML);
	}

	/**
	 * Returns the x position offset of the label relative to the tick position
	 * on the axis.
	 * <p> Default: 0
	 */
	public Number getX() {
		return getAttr(Attrs.x, 0).asNumber();
	}

	/**
	 * Sets the x position offset of the label relative to the tick position on
	 * the axis.
	 * <p> Default: 0
	 */
	public void setX(Number x) {
		setAttr(Attrs.x, x, 0);
	}

	/**
	 * Returns the y position offset of the label relative to the tick position
	 * on the axis.
	 * <p> Default: 0
	 */
	public Number getY() {
		return getAttr(Attrs.y, 0).asNumber();
	}

	/**
	 * Sets the y position offset of the label relative to the tick position on
	 * the axis.
	 * 
	 * <p> Default: 0
	 */
	public void setY(Number y) {
		// we don't use 0 here to solve HC JS inconsistent issue
		setAttr(Attrs.y, y, NOT_NULL_VALUE);
	}

	/**
	 * Returns the Z index for the axis labels.
	 * <p>
	 * Default: 7.
	 */
	public Number getZIndex() {
		return getAttr(Attrs.zIndex, 7).asNumber();
	}

	/**
	 * Sets the Z index for the axis labels.
	 */
	public void setZIndex(Number zIndex) {
		setAttr(Attrs.zIndex, zIndex, 7);
	}

}