/* TreemapPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Fri, Oct  2, 2015 11:13:24 AM, Created by Christopher

Copyright (C) 2015 Potix Corporation. All Rights Reserved.

This program is distributed under LGPL Version 2.1 in the hope that
it will be useful, but WITHOUT ANY WARRANTY.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;

import org.zkoss.chart.Charts;
import org.zkoss.chart.Color;
import org.zkoss.chart.Level;
import org.zkoss.chart.LinearGradient;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.RadialGradient;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * Options for treemap.
 * @author Christopher
 */
public class TreemapPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		allowDrillToNode,
		alternateStartingDirection,
		borderColor,
		borderWidth,
		colorByPoint,
		colors,
		interactByLeaf,
		layoutAlgorithm,
		layoutStartingDirection,
		levelIsConstant,
		levels
	}
	
	/**
	 * When enabled the user can click on a point which is a parent and zoom in
	 * on its children.
	 * <p>
	 * Defaults to false.
	 * @since 2.1.0
	 */
	public boolean getAllowDrillToNode() {
		return getAttr(Attrs.allowDrillToNode, false).asBoolean();
	}
	/**
	 * @param allowDrillToNode
	 * @see #getAllowDrillToNode()
	 * @since 2.1.0
	 */
	public void setAllowDrillToNode(boolean allowDrillToNode) {
		setAttr(Attrs.allowDrillToNode, allowDrillToNode, false);
	}
	
	/**
	 * Enabling this option will make the treemap alternate the drawing
	 * direction between vertical and horizontal. The next levels starting
	 * direction will always be the opposite of the previous.
	 * <p>
	 * Defaults to false.
	 * @since 2.1.0
	 */
	public boolean getAlternateStartingDirection() {
		return getAttr(Attrs.alternateStartingDirection, false).asBoolean();
	}
	/**
	 * @param alternateStartingDirection
	 * @see #getAlternateStartingDirection()
	 * @since 2.1.0
	 */
	public void setAlternateStartingDirection(boolean alternateStartingDirection) {
		setAttr(Attrs.alternateStartingDirection, alternateStartingDirection, false);
	}
	
	/**
	 * The color of the border surrounding each tree map item.
	 * <p>
	 * Defaults to #E0E0E0.
	 * @since 2.1.0
	 */
	public Color getBorderColor() {
		if (!containsKey(Attrs.borderColor)) {
			setBorderColor("#E0E0E0");
		}
		return (Color) getAttr(Attrs.borderColor);
	}
	/**
	 * Sets the color of the border surrounding each column or bar.
	 * @see #getBorderColor()
	 * @see Color
	 * @since 2.1.0
	 */
	public void setBorderColor(Color color) {
		setAttr(Attrs.borderColor, color, NOT_NULL_VALUE);
	}
	/**
	 * Sets the color of the border surrounding each column or bar.
	 * @see #getBorderColor()
	 * @since 2.1.0
	 */
	public void setBorderColor(String color) {
		setBorderColor(new Color(color));
	}
	/**
	 * Sets the color of the border surrounding each column or bar.
	 * @see #getBorderColor()
	 * @see LinearGradient
	 * @since 2.1.0
	 */
	public void setBorderColor(LinearGradient color) {
		setBorderColor(new Color(color));
	}
	/**
	 * Sets the color of the border surrounding each column or bar.
	 * @see #getBorderColor()
	 * @see RadialGradient
	 * @since 2.1.0
	 */
	public void setBorderColor(RadialGradient color) {
		setBorderColor(new Color(color));
	}
	
	/**
	 * Returns the width of the border surrounding each column or bar.
	 * <p>
	 * Default: 1.
	 * @return width of the border surrounding each column or bar.
	 * @since 2.1.0
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, 1).asNumber();
	}
	/**
	 * Sets the width of the border surrounding each column or bar.
	 * @since 2.1.0
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth, 1);
	}
	
	/**
	 * Returns whether color by point option is enabled or not.
	 * This option determines whether the chart should receive one color
	 * per series or one color per point
	 * 
	 * Default: false.
	 * @since 2.1.0
	 */
	public boolean isColorByPoint() {
		return getAttr(Attrs.colorByPoint, false).asBoolean();
	}
	/**
	 * Sets whether color by point option is enabled or not.
	 * This option determines whether the chart should receive one color
	 * per series or one color per point
	 * @since 2.1.0
	 */
	public void setColorByPoint(boolean colorByPoint) {
		setAttr(Attrs.colorByPoint, colorByPoint, false);
	}
	
	/**
	 * Returns a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 2.1.0
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}
	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 2.1.0
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}
	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 2.1.0
	 */
	public void setColors(String... source) {
		Color[] colors = new Color[source.length];
		int i = 0;
		for (String s : source)
			colors[i++] = new Color(s);
		setColors(Arrays.asList(colors));
	}
	
	/**
	 * This option decides if the user can interact with the parent nodes or 
	 * just the leaf nodes. When this option is undefined, it will be true by 
	 * default. However when allowDrillToNode is true, 
	 * then it will be false by default.
	 * @since 2.1.0
	 */
	public boolean getInteractByLeaf() {
		if ((Boolean) getAttr(Attrs.allowDrillToNode)) {
			return getAttr(Attrs.interactByLeaf, false).asBoolean();
		} else {
			return getAttr(Attrs.interactByLeaf, true).asBoolean();
		}
	}
	/**
	 * @param interactByLeaf
	 * @see #getInteractByLeaf()
	 * @since 2.1.0
	 */
	public void setInteractByLeaf(boolean interactByLeaf) {
		setAttr(Attrs.interactByLeaf, interactByLeaf);
	}
	
	/**
	 * Get the option that decides which algorithm is used for setting position and dimensions of the points.
	 * <p>
	 * Default: "sliceAndDice"
	 * @return either "sliceAndDice", "stripes", "squarified", or "strip"
	 * @since 2.1.0
	 */
	public String getLayoutAlgorithm() {
		return getAttr(Attrs.layoutAlgorithm, "sliceAndDice").asString();
	}
	/**
	 * Set the option decides which algorithm is used for setting position and dimensions of the points.
	 * <p>
	 * @param layoutAlgorithm
	 * @since 2.1.0
	 */
	public void setLayoutAlgorithm(String layoutAlgorithm) {
		if (!("sliceAndDice".equals(layoutAlgorithm) ||
			"stripes".equals(layoutAlgorithm) ||
			"squarified".equals(layoutAlgorithm) ||
			"strip".equals(layoutAlgorithm))) {
			throw new IllegalArgumentException("Unsupported layout algorithm: [" + layoutAlgorithm + "]");
		}
		setAttr(Attrs.layoutAlgorithm, layoutAlgorithm, "sliceAndDice");
	}
	
	/**
	 * Defines which direction the layout algorithm will start drawing.
	 * <p>
	 * Default: "vertical"
	 * @return either "vertical" or "horizontal
	 * @since 2.1.0
	 */
	public String getLayoutStartingDirection() {
		return getAttr(Attrs.layoutStartingDirection, "vertical").asString();
	}
	/**
	 * Set which direction the layout algorithm will start drawing.
	 * <p>
	 * Possible values are "vertical" and "horizontal".
	 * @param layoutStartingDirection
	 * @see #getLayoutStartingDirection()
	 * @since 2.1.0
	 */
	public void setLayoutStartingDirection(String layoutStartingDirection) {
		if (!("vertical".equalsIgnoreCase(layoutStartingDirection) || "horizontal".equalsIgnoreCase(layoutStartingDirection))) {
			throw new IllegalArgumentException("Unsopported layout starting direction:[" + layoutStartingDirection + "]");
		}
		setAttr(Attrs.layoutStartingDirection, layoutStartingDirection, "vertical");
	}
	
	/**
	 * Used together with the levels and allowDrillToNode options. When set to
	 * false the first level visible when drilling is considered to be level
	 * one. Otherwise the level will be the same as the tree structure.
	 * <p>
	 * Defaults to true.
	 * @since 2.1.0
	 */
	public boolean getLevelIsConstant() {
		return getAttr(Attrs.levelIsConstant, true).asBoolean();
	}
	/**
	 * @param levelIsConstant
	 * @see #getLevelIsConstant()
	 * @since 2.1.0
	 */
	public void setLevelIsConstant(boolean levelIsConstant) {
		setAttr(Attrs.levelIsConstant, levelIsConstant, true);
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not point options.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public List<Level> getLevels() {
		return Generics.cast(getAttr(Attrs.levels, null).asValue());
	}
	/**
	 * @param levels
	 * @see #getLevels()
	 * @since 2.1.0
	 */
	public void setLevels(List<Level> levels) {
		setAttr(Attrs.levels, levels);
	}
}
